/******************************************************************************
*  Nano-RK, a real-time operating system for sensor networks.
*  Copyright (C) 2007, Real-Time and Multimedia Lab, Carnegie Mellon University
*  All rights reserved.
*
*  This is the Open Source Version of Nano-RK included as part of a Dual
*  Licensing Model. If you are unsure which license to use please refer to:
*  http://www.nanork.org/nano-RK/wiki/Licensing
*
*  This program is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, version 2.0 of the License.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include <nrk.h>
#include <include.h>
#include <ulib.h>
#include <stdio.h>
#include <hal.h>
#include <nrk_error.h>
#include <nrk_timer.h>
#include <nrk_stack_check.h>
#include <nrk_stats.h>
#include <tmote_basic_sensor.h>
#include <basic_rf.h>




NRK_STK Stack1[NRK_APP_STACKSIZE];
nrk_task_type TaskOne;
void Task1(void);

RF_TX_INFO rfTxInfo;
RF_RX_INFO rfRxInfo;
uint8_t tx_buf[RF_MAX_PAYLOAD_SIZE];
uint8_t rx_buf[RF_MAX_PAYLOAD_SIZE];

NRK_STK Stack1[NRK_APP_STACKSIZE];
nrk_task_type TaskOne;
void Task1(void);

NRK_STK Stack2[NRK_APP_STACKSIZE];
nrk_task_type TaskTwo;
void Task2(void);


void nrk_create_taskset();
uint8_t kill_stack(uint8_t val);

int main ()
{
  nrk_setup_ports();
  nrk_setup_uart(UART_BAUDRATE_115K2);

  nrk_init();
  enable_user_button();
  nrk_kprintf( PSTR("Nano-RK Version ") );
  printf( "%d\r\n",NRK_VERSION );
  printf( "main\n" );
  nrk_led_clr(BLUE_LED);
  nrk_led_clr(GREEN_LED);
  nrk_led_clr(RED_LED);

  nrk_time_set(0,0);
  nrk_create_taskset ();

  nrk_start();

  return 0;
}

void Task1()
{
  nrk_time_t t;
  uint16_t cnt;
  uint8_t i,payload;
  cnt=0;
  nrk_kprintf( PSTR("Nano-RK Version ") );
  printf( "%d\r\n",NRK_VERSION );

  printf( "My node's address is %u\r\n",NODE_ADDR );
  printf( "Task1 PID=%u\r\n",nrk_get_pid());
  rf_init (&rfRxInfo, 25, 0x2420, 0x1215);
  cnt=0;
  while(1)
    {
             if(BUTTON_STATUS == B_ON)
              {  
    		rfTxInfo.pPayload=&payload;
                payload = cnt%2;  
	        printf( "Task1 cnt=%u\r\n",cnt );
                rfTxInfo.length=sizeof(payload);
		rf_tx_packet (&rfTxInfo);
                if(payload)	
         	   nrk_led_set(RED_LED);
                else
         	   nrk_led_clr(RED_LED);
              }
               else 
         	   nrk_led_clr(RED_LED);
              
		cnt++;
	       nrk_wait_until_next_period();
    }
}
void Task2()
{
    uint8_t cnt,i,length;
    int8_t n ;
    rfRxInfo.pPayload = rx_buf;
    rfRxInfo.max_length = RF_MAX_PAYLOAD_SIZE;
    rfRxInfo.ackRequest= 0;
    rf_init (&rfRxInfo, 25, 0x2420, 0x1215);
    printf( "Waiting for packet...\r\n" );
    rf_set_rx (&rfRxInfo, 25); 	
    rf_polling_rx_on ();
    while(1){
	if((n = rf_rx_check_fifop()) == 0){
 		//nrk_led_clr(RED_LED);	
		nrk_wait_until_next_period();
	}	
	if (n != 0) {
        	n = 0;
        	cnt=0;
		while (  (n = rf_polling_rx_packet ()) == 0) {
			printf("Packet recieved\n\r");		
			if (cnt > 50) {
                		printf( "PKT Timeout\r\n" );
				break;          // huge timeout as failsafe
				}
			cnt++;
		}
		
    	}
    	if (n == 1) {
	if(rx_buf[0] == 1)
 		nrk_led_set(RED_LED);	
        else 
 		nrk_led_clr(RED_LED);	
		printf( "\r\n" );
    	} 
	nrk_wait_until_next_period();
     }
}

void
nrk_create_taskset()
{
if(NODE_ADDR == 1) {
  nrk_task_set_entry_function( &TaskOne, Task1);
  nrk_task_set_stk( &TaskOne, Stack1, NRK_APP_STACKSIZE);
  TaskOne.prio = 1;
  TaskOne.FirstActivation = TRUE;
  TaskOne.Type = BASIC_TASK;
  TaskOne.SchType = PREEMPTIVE;
  TaskOne.period.secs = 1;
  TaskOne.period.nano_secs = 0*NANOS_PER_MS;
  TaskOne.cpu_reserve.secs = 0;
  TaskOne.cpu_reserve.nano_secs = 100*NANOS_PER_MS;
  TaskOne.offset.secs = 0;
  TaskOne.offset.nano_secs= 0;
  nrk_activate_task (&TaskOne);

}else{

  nrk_task_set_entry_function( &TaskTwo, Task2);
  nrk_task_set_stk( &TaskTwo, Stack2, NRK_APP_STACKSIZE);
  TaskTwo.prio = 1;
  TaskTwo.FirstActivation = TRUE;
  TaskTwo.Type = BASIC_TASK;
  TaskTwo.SchType = PREEMPTIVE;
  TaskTwo.period.secs = 0;
  TaskTwo.period.nano_secs = 20*NANOS_PER_MS;
  TaskTwo.cpu_reserve.secs = 0;
  TaskTwo.cpu_reserve.nano_secs = 19*NANOS_PER_MS;
  TaskTwo.offset.secs = 0;
  TaskTwo.offset.nano_secs= 0;
  nrk_activate_task (&TaskTwo);
}
  return;
}

uint8_t kill_stack(uint8_t val)
{
  char bad_memory[10];
  uint8_t i;

  for(i=0; i<10; i++ ) bad_memory[i]=i;
  for(i=0; i<10; i++ ) printf( "%d ", bad_memory[i]);
  printf( "Die Stack %d\r\n",val );
  if(val>1) kill_stack(val-1);
  return 0;
}
