/******************************************************************************
*  Nano-RK, a real-time operating system for sensor networks.
*  Copyright (C) 2007, Real-Time and Multimedia Lab, Carnegie Mellon University
*  All rights reserved.
*
*  This is the Open Source Version of Nano-RK included as part of a Dual
*  Licensing Model. If you are unsure which license to use please refer to:
*  http://www.nanork.org/nano-RK/wiki/Licensing
*
*  This program is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, version 2.0 of the License.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*  Contributing Authors (specific to this file):
*  Zane Starr
*******************************************************************************/



#include <nrk.h>
#include <include.h>
#include <ulib.h>
#include <stdio.h>
#include <hal.h>
#include <nrk_error.h>
#include <nrk_timer.h>
#include <nrk_driver_list.h>
#include <nrk_driver.h>
#include <tmote_basic_sensor.h>
#include "nrk_cfg.h"
#include "msp-adc.h"

#define threshold  5

NRK_STK Stack1[NRK_APP_STACKSIZE];
nrk_task_type TaskOne;
void Task1(void);

NRK_STK Stack2[NRK_APP_STACKSIZE];
nrk_task_type TaskTwo;
void Task2(void);

void nrk_create_taskset();
void nrk_register_drivers();
uint8_t kill_stack(uint8_t val);

uint16_t dark;

int main ()
{
  uint8_t t;
  nrk_setup_ports();
  nrk_setup_uart(UART_BAUDRATE_115K2);

  printf( "Starting up...\r\n" );

  nrk_init();
  nrk_time_set(0,0);
  enable_user_button(); 
  nrk_led_clr(RED_LED); 
  nrk_led_clr(GREEN_LED); 
  nrk_led_clr(BLUE_LED); 
  nrk_register_drivers();
  nrk_create_taskset ();
  nrk_start();
     
  return 0;
}


void Task1()
{
uint16_t cnt;
int8_t i,fd,val;
uint16_t buf;
  printf( "My node's address is %d\r\n",NODE_ADDR );

  printf( "Task1 PID=%d\r\n",nrk_get_pid());

  fd=nrk_open(TMOTE_BASIC_SENSOR,READ);	
  val=nrk_set_status(fd,SENSOR_SELECT,LIGHT);
  cnt=0;
  while(1) {



  	if(cnt < 20 ){
                if(cnt>15) 
	   {	val = nrk_read(fd,&buf,2);
           dark+=buf;
          }
	}
        else if(cnt == 20)
        {
              dark/=4;

         }
         else{

   		val = nrk_read(fd,&buf,2);
	if(buf > dark + threshold){
		nrk_led_set(RED_LED);
	}
        else 
		nrk_led_clr(RED_LED);
        }
        printf(" lt2 %u\n\r",buf);	
	nrk_wait_until_next_period();
	cnt++;
 	
       }
}
void Task2()
{
uint16_t cnt;
int8_t i,fd,val;
uint16_t buf;
  printf( "My node's address is %d\r\n",NODE_ADDR );

  printf( "Task1 PID=%d\r\n",nrk_get_pid());

  cnt=0;
  while(1) {
	if(BUTTON_STATUS == B_ON)
        {
        // if(cnt%2)
	  nrk_led_toggle(RED_LED);
        // else
	  //nrk_led_clr(RED_LED);

	}
        else
	  nrk_led_clr(RED_LED);
            
        printf("count %u\r\n",cnt); 
	nrk_wait_until_next_period();
	cnt++;
       }
}


void
nrk_create_taskset()
{
  if(NODE_ADDR == 1){
  TaskOne.task = Task1;
  nrk_task_set_stk( &TaskOne, Stack1, NRK_APP_STACKSIZE);
  TaskOne.prio = 1;
  TaskOne.FirstActivation = TRUE;
  TaskOne.Type = BASIC_TASK;
  TaskOne.SchType = PREEMPTIVE;
  TaskOne.period.secs = 0;
  TaskOne.period.nano_secs = 200*NANOS_PER_MS; //*NANOS_PER_MS;
  TaskOne.cpu_reserve.secs = 0;
  TaskOne.cpu_reserve.nano_secs =  100*NANOS_PER_MS;
  TaskOne.offset.secs = 0;
  TaskOne.offset.nano_secs= 0;
  nrk_activate_task (&TaskOne);
 }else{ 
  TaskTwo.task = Task2;
  nrk_task_set_stk( &TaskTwo, Stack2, NRK_APP_STACKSIZE);
  TaskTwo.prio = 1;
  TaskTwo.FirstActivation = TRUE;
  TaskTwo.Type = BASIC_TASK;
  TaskTwo.SchType = PREEMPTIVE;
  TaskTwo.period.secs = 0;
  TaskTwo.period.nano_secs = 500*NANOS_PER_MS; //*NANOS_PER_MS;
  TaskTwo.cpu_reserve.secs = 0;
  TaskTwo.cpu_reserve.nano_secs =  200*NANOS_PER_MS;
  TaskTwo.offset.secs = 0;
  TaskTwo.offset.nano_secs= 0;
  nrk_activate_task (&TaskTwo);
}
}

void nrk_register_drivers()
{
int8_t val;

// Register the Basic Tmote Sensor device driver
// Make sure to add: 
//     #define NRK_MAX_DRIVER_CNT  
//     in nrk_cfg.h
// Make sure to add: 
//     SRC += $(ROOT_DIR)/src/drivers/platform/$(PLATFORM_TYPE)/source/tmote_basic_sensor.c
//     in makefile
	val=nrk_register_driver( &dev_manager_tmote_sensors,TMOTE_BASIC_SENSOR);
	if(val==NRK_ERROR) nrk_kprintf( PSTR("Failed to load  driver\r\n") );

}


