/******************************************************************************
*  Nano-RK, a real-time operating system for sensor networks.
*  Copyright (C) 2007, Real-Time and Multimedia Lab, Carnegie Mellon University
*  All rights reserved.
*
*  This is the Open Source Version of Nano-RK included as part of a Dual
*  Licensing Model. If you are unsure which license to use please refer to:
*  http://www.nanork.org/nano-RK/wiki/Licensing
*
*  This program is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, version 2.0 of the License.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

/*
 * Contributing authors (this file):
 * Anthony Rowe
 * Mark Hamilton
 */

#include <nrk.h>
#include <include.h>
#include <ulib.h>
#include <stdio.h>
#include <hal.h>
#include <nrk_error.h>
#include <nrk_timer.h>
#include <nrk_stack_check.h>

NRK_STK Stack1[NRK_APP_STACKSIZE];
nrk_task_type TaskOne;
void Task1(void);

NRK_STK Stack2[NRK_APP_STACKSIZE];
nrk_task_type TaskTwo;
void Task2(void);

void nrk_create_taskset();

int
main ()
{
	nrk_init_hardware();
	nrk_setup_ports();
	nrk_setup_uart(UART_BAUDRATE_115K2);

	nrk_init();
	nrk_time_set(0,0);
	nrk_create_taskset();

	printf("Starting up...\r\n");

	nrk_start();

	return 0;
}

void Task1()
{
	while(1) {
		nrk_gpio_toggle(NRK_LED);
		printf("Task 1\r\n");
		nrk_wait_until_next_period();
	}
}

void Task2()
{
	while(1) {
		nrk_gpio_toggle(NRK_MISC2);
		printf("Task 2\r\n");
		nrk_wait_until_next_period();
	}
}

void
nrk_create_taskset()
{
	nrk_task_set_entry_function(&TaskOne, Task1);
	TaskOne.Ptos = (void *) &Stack1[NRK_APP_STACKSIZE];
	TaskOne.Pbos = (void *) &Stack1[0];
	TaskOne.prio = 1;
	TaskOne.FirstActivation = TRUE;
	TaskOne.Type = BASIC_TASK;
	TaskOne.SchType = PREEMPTIVE;
	TaskOne.period.secs = 0;
	TaskOne.period.nano_secs = 600*NANOS_PER_MS;
	TaskOne.cpu_reserve.secs = 0;
	TaskOne.cpu_reserve.nano_secs =  50*NANOS_PER_MS;
	TaskOne.offset.secs = 0;
	TaskOne.offset.nano_secs = 0;
	nrk_activate_task (&TaskOne);

	nrk_task_set_entry_function(&TaskTwo, Task2);
	TaskTwo.Ptos = (void *) &Stack2[NRK_APP_STACKSIZE];
	TaskTwo.Pbos = (void *) &Stack2[0];
	TaskTwo.prio = 2;
	TaskTwo.FirstActivation = TRUE;
	TaskTwo.Type = BASIC_TASK;
	TaskTwo.SchType = PREEMPTIVE;
	TaskTwo.period.secs = 0;
	TaskTwo.period.nano_secs = 600*NANOS_PER_MS;
	TaskTwo.cpu_reserve.secs = 0;
	TaskTwo.cpu_reserve.nano_secs =  50*NANOS_PER_MS;
	TaskTwo.offset.secs = 0;
	TaskTwo.offset.nano_secs = 0;
	nrk_activate_task (&TaskTwo);
}

