/******************************************************************************
*  Nano-RK, a real-time operating system for sensor networks.
*  Copyright (C) 2007, Real-Time and Multimedia Lab, Carnegie Mellon University
*  All rights reserved.
*
*  This is the Open Source Version of Nano-RK included as part of a Dual
*  Licensing Model. If you are unsure which license to use please refer to:
*  http://www.nanork.org/nano-RK/wiki/Licensing
*
*  This program is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, version 2.0 of the License.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*  Contributing Authors (specific to this file):
*  Zane Starr
*  Anthony Rowe
*******************************************************************************/

#ifndef __TMOTE_BASIC_SENSOR_H__
#define __TMOTE_BASIC_SENSOR_H__

#include<stdio.h>

// GET/SET Status Options
#define SENSOR_SELECT   1

// SET_SENSOR options
#define TEMP	        1
#define HUMIDITY	2
#define LIGHT 		3

#define DEV_MODE_ON     1
#define DEV_MODE_OFF    2
#define DEV_MODE_IDLE   3 


/* Pin definitions */
#define SET_DIR1(port, pin)    (P##port##DIR |= (1 << pin))
#define UNSET_DIR1(port, pin)    (P##port##DIR &= ~(1 << pin))

#define SET_PORT1(port, pin)    (P##port##OUT |= (1 << pin))
#define UNSET_PORT1(port, pin)    (P##port##OUT &= ~(1 << pin))

#define PORT_IN1(port)  (P##port##IN)

#define UNSET_DIR_(p,q) UNSET_DIR1(p,q)
#define SET_DIR_(p,q) SET_DIR1(p,q)
#define PORT_IN(p) PORT_IN1(p)
#define SET_PORT_(p,q) SET_PORT1(p,q)
#define UNSET_PORT_(p,q) UNSET_PORT1(p,q)


#define UNSET_PORT(p) UNSET_PORT_(p)
#define UNSET_DIR(p) UNSET_DIR_(p)
#define SET_DIR(p)    SET_DIR_(p)
#define SET_PORT(p)  SET_PORT_(p)

#define SHT11_POWER_PORT 1
#define SHT11_POWER_PIN  7
#define SHT11_POWER  SHT11_POWER_PORT, SHT11_POWER_PIN

#define SHT11_SCLK_PORT  1
#define SHT11_SCLK_PIN   6
#define SHT11_SCLK  SHT11_SCLK_PORT, SHT11_SCLK_PIN

#define SHT11_DATA_PORT  1
#define SHT11_DATA_PIN   5
#define SHT11_DATA  SHT11_DATA_PORT, SHT11_DATA_PIN

/* Command bytes */
#define SHT11_CMD_READ_TEMP     3 
#define SHT11_CMD_READ_HUMIDITY 5
#define SHT11_CMD_READ_SR       7
#define SHT11_CMD_WRITE_SR      6
#define SHT11_CMD_SOFT_RESET    30

/* Status register masks */
#define SHT11_SR_HEATER  (1 << 2)
#define SHT11_SR_RELOAD  (1 << 1)
#define SHT11_SR_LOW_RES (1 << 0)


/* arguments for dev_ioctl_MSP_HUMIDITY/TEMPERATURE *
#define SHT11_LOW_RES_ON   0x01
#define SHT11_LOW_RES_OFF  0x02
#define SHT11_HEATER_ON    0x04
#define SHT11_HEATER_OFF   0x08
#define SHT11_RELOAD_ON    0x10
#define SHT11_RELOAD_OFF   0x20*/

#define SENSOR_SETUP_DELAY  11000


#define DATA_AS_OUT() SET_DIR(SHT11_DATA)
#define DATA_AS_IN()  UNSET_DIR(SHT11_DATA)

#define SCLK_HI()     SET_PORT(SHT11_SCLK)
#define SCLK_LO()     UNSET_PORT(SHT11_SCLK)

#define DATA_HI()     SET_PORT(SHT11_DATA)
#define DATA_LO()     UNSET_PORT(SHT11_DATA)

#define B_ON 1
#define B_OFF 0

#define adc_on()    /* turn on reference generator */	\
   ADC12CTL0 |= REF2_5V | REFON;			\
   /* turn on ADC12 */					\
   ADC12CTL0 |= ADC12ON;				\
   /* enable conversion to take place */		\
   /* ADC12CTL0 |= ENC; */


#define adc_off()    /* turn off reference voltage */	\
   /* ADC12CTL0 &= ~REFON; */				\
   /* turn off adc core */	 			\
   /* ADC12CTL0 &= ~ADC12ON; */
/* Function declarations */



extern uint8_t BUTTON_STATUS ; 




void delay();
uint8_t dev_manager_tmote_sensors(uint8_t state,uint8_t opt,uint8_t * buffer,uint8_t size);
void sht11_init(void);
#endif
