/******************************************************************************
*  Nano-RK, a real-time operating system for sensor networks.
*  Copyright (C) 2007, Real-Time and Multimedia Lab, Carnegie Mellon University
*  All rights reserved.
*
*  This is the Open Source Version of Nano-RK included as part of a Dual
*  Licensing Model. If you are unsure which license to use please refer to:
*  http://www.nanork.org/nano-RK/wiki/Licensing
*
*  This program is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, version 2.0 of the License.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*  Contributing Authors (specific to this file):
*  Anthony Rowe
*  Zane Starr
*  Anand Eswaren
*******************************************************************************/

#include <include.h>
#include <nrk.h>
#include <nrk_stack_check.h>
#include <nrk.h>
#include <nrk_task.h>
#include <nrk_defs.h>
#include <nrk_cfg.h>
#include <nrk_timer.h>        //paja
#include <nrk_error.h>

#define BUILD_DATE "Date: " __DATE__ "\n"

/*
*********************************************************************************************************
*                                        INITIALIZE A TASK'S STACK
*
* Description: This function is highly processor specific.
*
* Arguments  : task          is a pointer to the task code
*
*              pdata         is a pointer to a user supplied data area that will be passed to the task
*                            when the task first executes.
*
*              ptos          is a pointer to the top of stack.  It is assumed that 'ptos' points to
*                            a 'free' entry on the task stack.  
*                            'ptos' contains the HIGHEST valid address of the stack.  
*
*              opt           specifies options that can be used to alter the behavior of OSTaskStkInit().
*                            We don't use have any option implemented for this project. You can just
*                            set opt to 0
*
* Returns    : Always returns the location of the new top-of-stack' once the processor registers have
*              been placed on the stack in the proper order.
*
* Note(s)    : 
*********************************************************************************************************
*/


void nrk_battery_save()
{
/*
#ifdef NRK_BATTERY_SAVE
 	_nrk_stop_os_timer();
        _nrk_set_next_wakeup(250);
        nrk_led_clr(0);
        nrk_led_set(1);
        nrk_led_clr(2);
        nrk_led_clr(3);
        SET_VREG_INACTIVE();
        nrk_sleep();
#endif
*/
}


void nrk_task_set_entry_function( nrk_task_type *task, void *func )
{
		task->task = func;
}

//void nrk_task_set_stk( nrk_task_type *task, void *stk_base, uint16_t stk_size )
void nrk_task_set_stk( nrk_task_type *task, NRK_STK stk_base[], uint16_t stk_size )
{
	void* addr;
	if(stk_size<32) nrk_error_add(NRK_STACK_TOO_SMALL);

	addr=&(stk_base[stk_size-2]);
	task->Ptos = addr;
	addr=&(stk_base[0]);
	task->Pbos = addr;
}

void *nrk_task_stk_init (void (*task)(), void *ptos, void *pbos)
{
    uint16_t *stk ;  // 2 bytes
    uint8_t *stkc; // 1 byte

    stk    = (unsigned int *)pbos;          /* Load stack pointer */ 
    stkc = (unsigned char*)stk;
    *stkc = STK_CANARY_VAL;  // Flag for Stack Overflow    
    stk    = (unsigned int *)ptos;          /* Load stack pointer */
    /* build a context for the new task */
    --stk;
    *stk = task;
    *(--stk) = GIE; //enable interrupts
    *(--stk) = 0;       
    *(--stk) = 0;                        
    *(--stk) = 0;                        
    *(--stk) = 0;                         
    *(--stk) = 0;                         
    *(--stk) = 0;                        
    *(--stk) = 0; 

    *(--stk) = 0; 
    *(--stk) = 0; 
    *(--stk) = 0; 
    *(--stk) = 0; 
    *(--stk) = 0; 

    return ((void *)stk);
}

inline void nrk_stack_pointer_init()
{
	uint16_t *stk;
	uint8_t *stkc;
#ifdef KERNEL_STK_ARRAY
        stk = (uint16_t*)&nrk_kernel_stk[NRK_KERNEL_STACKSIZE-1];
        nrk_kernel_stk[0]=STK_CANARY_VAL;
        nrk_kernel_stk_ptr = &nrk_kernel_stk[NRK_KERNEL_STACKSIZE-1];
#else
        stk = (uint16_t*)NRK_KERNEL_STK_TOP;
	stkc = (unsigned char*)(NRK_KERNEL_STK_TOP-NRK_KERNEL_STACKSIZE);
	*stkc = STK_CANARY_VAL;  // Flag for Stack Overflow    
        nrk_kernel_stk_ptr = (uint16_t*)NRK_KERNEL_STK_TOP;
#endif

	*stk = (uint16_t)_nrk_timer_tick;
}


inline void nrk_stack_pointer_restore()
{
//unsigned char *stkc;
uint16_t *stkc;

#ifdef KERNEL_STK_ARRAY
        stkc = (uint16_t*)&nrk_kernel_stk[NRK_KERNEL_STACKSIZE-1];
#else
        stkc = (uint16_t*)NRK_KERNEL_STK_TOP;
#endif
	*stkc = (uint16_t)_nrk_timer_tick;
}

/* start the target running */
void nrk_target_start(void)
{

  _nrk_setup_timer();
  nrk_int_enable();  
	
}
void nrk_sleep()
{
//	nrk_led_toggle(RED_LED);	
	TBR = 0;
	TBCCR0  = _nrk_get_next_wakeup() ;
	TBCTL |= MC_UPTO_CCR0;
	eint();
	_BIS_SR(LPM3_bits);
}


void sleep_timer_init(void)
{
   TBCTL = TBCLR;
   // src = ACLK, DIV = 8, INT = enabled, 16bit
   TBCTL = TBSSEL_ACLK | ID_DIV4 | TBIE | CNTL_0;

   //last_sleep_time  = 0;
}

interrupt (TIMERB1_VECTOR) sleep_wake_int()
{   if(TBIV == 14)
    {	
	TBCTL &= ~(MC_3);
	_BIC_SR_IRQ(LPM4_bits);
	 //nrk_led_toggle(RED_LED);	
			
		
    }
    
}
void nrk_idle(){


}

