/*****************************************************************************
* Copyright (c) 2007, Real-Time and Multimedia Lab, Carnegie Mellon University
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of Carnegie Mellon University nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY CARNEGIE MELLON UNIVERSITY ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* Contributing Authors:
* Anthony Rowe
*******************************************************************************/

//#define __MSP430_HAS_BC2__
#include <include.h>
//#include <avr/interrupt.h>
#include <signal.h>
#include <ulib.h>
#include <nrk_scheduler.h>
#include <nrk_timer.h>
#include <nrk_error.h>

void nrk_spin_wait_us(uint16_t timeout)
{
    // This sequence uses exactly 8 clock cycle for each round
    do {
        NOP();
        NOP();
        NOP();
        NOP();
    } while (--timeout);
}

void _nrk_setup_timer()
{
        _nrk_prev_timer_val = MAX_SCHED_WAKEUP_TIME - 1;
	BCSCTL1 |= DIVA_DIV8; // ACLK = LFXT1CLK / 8
	TACTL = TASSEL_1+MC_0+ID_2; // Timer_A clock src=ACLK/4 (ID_2) up mode
	TACCR0 = _nrk_prev_timer_val;
	TACCTL0 &= ~CCIFG; // Clear compare interrupt flag
	TACCTL0 |= CCIE; // Set compare interrupt enable / output flag

	_nrk_reset_os_timer();
	_nrk_start_os_timer();
	_nrk_time_trigger = 0;
}

void _nrk_high_speed_timer_stop()
{
	// TODO: implement
}

void _nrk_high_speed_timer_start()
{
	// TODO: implement
}


void _nrk_high_speed_timer_reset()
{
	// TODO: implement
}

/**
  This function blocks for n ticks of the high speed timer after the
  start number of ticks.  It will handle the overflow that can occur.
  Do not use this for delays longer than 8ms!
*/
void nrk_high_speed_timer_wait( uint16_t start, uint16_t ticks )
{
	// TODO: implement
}

inline uint16_t _nrk_high_speed_timer_get()
{
	// TODO: implement
	return 0;
}

void _nrk_stop_os_timer()
{
	TACTL &= (~(MC1 | MC0)); // Timer_A stop mode
}

void _nrk_start_os_timer()
{
	TAR = 0; // reset counter
	TACTL |= MC_1; // Up mode - timer counts to TACCR0
}

inline void _nrk_reset_os_timer()
{
	TAR = 0;
	_nrk_time_trigger = 0;
//	_nrk_prev_timer_val = 0; //already setup_timer above
}


uint16_t _nrk_get_next_wakeup()
{
	return TACCR0+1;
}

void _nrk_set_next_wakeup(uint16_t nw)
{
        TACCR0 = nw - 1;
	TACCTL0 &= ~CCIFG; // clear interrupt pending flag
	TACCTL0 |= CCIE; // Set compare interrupt enable / output flag
	TAR = 0;
}

inline uint16_t _nrk_os_timer_get()
{
    // CSS 10 Feb 2006: Brano Kusy notes MSP430 User's Guide, Section 11.2.1,
    // Note says reading a counter may return garbage if its clock source is
    // async.  The noted work around is to take a majority vote.

        uint16_t t0;
        uint16_t t1=TAR;
        do { t0=t1; t1=TAR; } while( t0 != t1 );
        return t1;
}

interrupt(TIMERA0_VECTOR) timer (void) {
	_nrk_timer_suspend_task();
}

//TODO: fix isrs for msp
/*
//-------------------------------------------------------------------------------------------------------
//  Default ISR 
//-------------------------------------------------------------------------------------------------------
interrupt(__vector_default) {
}                                  

#ifdef NRK_MAX_DRIVER_CNT
interrupt(SIG_OUTPUT_COMPARE3A)
{
}
#endif  
interrupt(TIMER2_OVF_vect) {
} 

interrupt(TIMER2_COMPA_vect) {
} 


//-------------------------------------------------------------------------------------------------------
//  TIMER 1 COMPARE ISR
//-------------------------------------------------------------------------------------------------------
interrupt(SIG_OUTPUT_COMPARE1A) {
} 
*/
