/***********************************************************************************
    Filename: hal_board.c

    Copyright 2007 Texas Instruments, Inc.
***********************************************************************************/
#include <include.h>
#include <ulib.h>
#include <stdio.h>
#include <hal.h>
#include <hal_tmote.h>
//#include <avr/interrupt.h>
#include <nrk_pin_define.h>
#include <nrk_error.h>


#define CLOCK_SPEED      (uint32_t)3686400

#define UBR0_INIT  0x40
#define UBR1_INIT  0x00
#define UMCTL_INIT 0x00


//------------------------------------------------------------------------------

// The constants below define some of the I/O signals used by the board
// Port, pin number, pin bitmask, direction and initial value should be
// set in order to match the target hardware. Once defined, the pins are
// configured in halBoardInit() by calling halDigioConfig()

#define ENABLE_GLOBAL_INT()         do { eint(); } while (0)


//------------------------------------------------------------------------------
//  void halBoardInit(void)
//
//  DESCRIPTION:
//    Set up board. Initialize MCU, configure I/O pins and user interfaces
//------------------------------------------------------------------------------
void halBoardInit(void)
{
    halAssyInit();
    //clock_init();
    nrk_int_enable();
}


void clock_init(void)
{
    uint16_t old_ccr2_val;
    int16_t new_ccr2_val;
    uint8_t rsel_val;
    uint8_t dco_val;

    BCSCTL1 = XT2OFF | DIVA1 | RSEL2 | RSEL0;
    BCSCTL2 = 0;
    //TACTL = TASSEL_2 | TACLR; // clear clock, source is SMCLK
    //TACTL |= MC1; // start timer in continuous mode
    CCTL2 = CM0 | CCIS0 | CAP; // rising edge capture

    for(;;) {
        while(!(CCTL2 & CCIFG))
            ; // wait for capture flag

        old_ccr2_val = CCR2;
        CCTL2 &= ~CCIFG;

        while(!(CCTL2 & CCIFG))
            ; // wait for capture flag
        CCTL2 &= ~CCIFG;

        rsel_val = BCSCTL1;
        rsel_val &= (RSEL0 | RSEL1 | RSEL2); // mask out rsel bits
        dco_val = DCOCTL;

        if(dco_val == 0xff) {
            if(rsel_val < 7) {
                BCSCTL1++;
            } else {
                goto err;
            }
        } else if(dco_val == 0) {
            if(rsel_val != 0) {
                BCSCTL1--;
            } else {
                goto err;
            }
        } else {
            new_ccr2_val = CCR2;
            new_ccr2_val -= old_ccr2_val;
            old_ccr2_val = CCR2;
            new_ccr2_val -= (CLOCK_SPEED / (32768 / 4));
            if(new_ccr2_val < 0) {
                DCOCTL++;
                continue;
            } else if(new_ccr2_val == 0) {
                goto err;
            } else {
                DCOCTL--;
                continue;
            }
        }

        DCOCTL = 0x60; // center dco
    }
err:
    CCTL2 = 0;
    BCSCTL1 &= ~(DIVA1);

    return;

}
